
#define WIN32_LEAN_AND_MEAN

// includes
#include <windows.h>
#include <sqlext.h>
#include <stdlib.h>
#include <stdio.h>
#include "DatabaseConnection.h"
#include "QueryResult.h"

/*------------------------------------
	Constructor
------------------------------------*/
DatabaseConnection::DatabaseConnection( )
{

	environment = NULL;
	connection = NULL;

	// create environment
	SQLRETURN return_code = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &environment );
	if( SQL_SUCCEEDED( return_code ) )
	{

		// set odbc version
		return_code = SQLSetEnvAttr( environment, SQL_ATTR_ODBC_VERSION, (SQLPOINTER*)SQL_OV_ODBC3, NULL );
		if( SQL_SUCCEEDED( return_code ) )
		{

			// bleh, nothing to do here yet

		}

	}

}

/*------------------------------------
	Constructor
------------------------------------*/
DatabaseConnection::DatabaseConnection( const char* connection_string )
{

	environment = NULL;
	connection = NULL;
	
	// create environment
	SQLRETURN return_code = SQLAllocHandle( SQL_HANDLE_ENV, SQL_NULL_HANDLE, &environment );
	if( SQL_SUCCEEDED( return_code ) )
	{

		// set odbc version
		return_code = SQLSetEnvAttr( environment, SQL_ATTR_ODBC_VERSION, (SQLPOINTER*)SQL_OV_ODBC3, NULL );
		if( SQL_SUCCEEDED( return_code ) )
		{

			// connect
			Connect( connection_string );

		}

	}

}


/*------------------------------------
	Destructor
------------------------------------*/
DatabaseConnection::~DatabaseConnection( )
{

	// disconnect
	Disconnect();

	// cleanup
	if( environment )
	{

		// free environment
		SQLFreeHandle( SQL_HANDLE_ENV, environment );
		environment = NULL;

	}


}

/*------------------------------------
	Connect
------------------------------------*/
bool DatabaseConnection::Connect( const char* connection_string )
{

	SQLCHAR buffer[ 1024 ];
	SQLSMALLINT buffer_size;

	// disconnect any old connections
	if( IsConnected() )
	{

		Disconnect();

	}

	// allocate a connection
	SQLRETURN return_code = SQLAllocHandle( SQL_HANDLE_DBC, environment, &connection );
	if( !SQL_SUCCEEDED( return_code ) )
	{

		// fail
		return false;

	}

	// set login timeout if applicable
	if( timeout_login > 0 )
	{

		// set
		return_code = SQLSetConnectAttr( connection, SQL_LOGIN_TIMEOUT, (SQLPOINTER)timeout_login, NULL );
		if( !SQL_SUCCEEDED( return_code ) )
		{

			// free the connection handle
			SQLFreeHandle( SQL_HANDLE_DBC, connection );

			// fail
			return false;

		}

	}

	// set connection timeout if applicable
	if( timeout > 0 )
	{

		// set
		return_code = SQLSetConnectAttr( connection, SQL_ATTR_CONNECTION_TIMEOUT, (SQLPOINTER)timeout, NULL );
		if( !SQL_SUCCEEDED( return_code ) )
		{

			// free the connection handle
			SQLFreeHandle( SQL_HANDLE_DBC, connection );

			// fail
			return false;

		}

	}

	// connect
	return_code = SQLDriverConnect(
		connection,
		NULL,
		(SQLCHAR*)connection_string,
		SQL_NTS,
		buffer,
		1024,
		&buffer_size,
		SQL_DRIVER_NOPROMPT

	);
	if( !SQL_SUCCEEDED( return_code ) )
	{

		// store last error
		if( return_code == SQL_ERROR || return_code == SQL_SUCCESS_WITH_INFO )
		{

			SQLCHAR sqlstate[ 6 ];
			SQLINTEGER error;
			SQLSMALLINT msg_length;

			// clear
			ZeroMemory( last_error, 512 );

			// get the first error
			return_code = SQLGetDiagRec(
				SQL_HANDLE_DBC,
				connection,
				1,
				sqlstate,
				&error,
				last_error, SQL_MAX_MESSAGE_LENGTH,
				&msg_length

			);

		}

		// free the connection handle
		SQLFreeHandle( SQL_HANDLE_DBC, connection );

		// fail
		return false;

	}

	// succeed
	return true;

}

/*------------------------------------
	Disconnect
------------------------------------*/
void DatabaseConnection::Disconnect( )
{

	// connection?
	if( connection )
	{

		// disconnect
		SQLDisconnect( connection );

		// free the connection handle
		SQLFreeHandle( SQL_HANDLE_DBC, connection );
		connection = NULL;

	}

}


/*------------------------------------
	SetTimeouts
------------------------------------*/
void DatabaseConnection::SetTimeouts( SQLINTEGER login_timeout, SQLINTEGER connection_timeout )
{

	timeout = connection_timeout;
	timeout_login = login_timeout;

}

/*------------------------------------
	Connected
------------------------------------*/
bool DatabaseConnection::IsConnected( )
{

	return connection != NULL;

}

/*------------------------------------
	ExecuteNoResult
------------------------------------*/
int DatabaseConnection::ExecuteNoResult( const char* sql )
{

	// valid connection?
	if( IsConnected() )
	{

		SQLHSTMT statement;
		SQLRETURN return_code;
		SQLINTEGER rows;

		// allocate a statement handle
		return_code = SQLAllocHandle( SQL_HANDLE_STMT, connection, &statement );
		if( SQL_SUCCEEDED( return_code ) )
		{

			// execute
			return_code = SQLExecDirect( statement, (SQLCHAR*)sql, SQL_NTS );
			if( SQL_SUCCEEDED( return_code ) )
			{

				// get the rows that were affected by this query
				SQLRowCount( statement, &rows );

				// free the statement
				SQLFreeHandle( SQL_HANDLE_STMT, statement );

				// return the affected rows
				return rows;

			}
			else
			{

				// store last error
				if( return_code == SQL_ERROR || return_code == SQL_SUCCESS_WITH_INFO )
				{

					SQLCHAR sqlstate[ 6 ];
					SQLINTEGER error;
					SQLSMALLINT msg_length;

					// clear
					ZeroMemory( last_error, 512 );

					// get the first error
					return_code = SQLGetDiagRec(
						SQL_HANDLE_STMT,
						statement,
						1,
						sqlstate,
						&error,
						last_error, SQL_MAX_MESSAGE_LENGTH,
						&msg_length

					);

				}

			}

			// free the statement
			SQLFreeHandle( SQL_HANDLE_STMT, statement );

		}
		

	}

	return 0;

}

/*------------------------------------
	Execute
------------------------------------*/
const char* DatabaseConnection::LastError( )
{

	return (const char*)last_error;

}


/*------------------------------------
	Execute
------------------------------------*/
QueryResult* DatabaseConnection::Execute( const char* sql )
{

	// valid connection?
	if( IsConnected() )
	{

		SQLHSTMT statement;
		SQLRETURN return_code;

		// allocate a statement handle
		return_code = SQLAllocHandle( SQL_HANDLE_STMT, connection, &statement );
		if( SQL_SUCCEEDED( return_code ) )
		{

			// execute
			return_code = SQLExecDirect( statement, (SQLCHAR*)sql, SQL_NTS );
			if( SQL_SUCCEEDED( return_code ) )
			{

				// setup a result
				return new QueryResult( statement );

			}
			else
			{

				// store last error
				if( return_code == SQL_ERROR || return_code == SQL_SUCCESS_WITH_INFO )
				{

					SQLCHAR sqlstate[ 6 ];
					SQLINTEGER error;
					SQLSMALLINT msg_length;

					// clear
					ZeroMemory( last_error, 512 );

					// get the first error
					return_code = SQLGetDiagRec(
						SQL_HANDLE_STMT,
						statement,
						1,
						sqlstate,
						&error,
						last_error, SQL_MAX_MESSAGE_LENGTH,
						&msg_length

					);

				}

			}

			// free the statement
			SQLFreeHandle( SQL_HANDLE_STMT, statement );

		}
		

	}

	return NULL;

}

